# -----------------------------------------------------------------------------
# Copyright (C) 2020  Jonathan Grahl <gn@igh.de>
#
# This file is part of cxmenu.
#
# Cxmenu is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Cxmenu is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with cxmenu.  If not, see <http://www.gnu.org/licenses/>.
# -----------------------------------------------------------------------------


# ---------------------------- Loading Own Modules ----------------------------
import curses
from curses import wrapper
from enum import Enum
import logging
import time as t

from cxmenutools import option as opt


# -----------------------------------------------------------------------------
class RemoteDisplay():
    """cxmenu application
    Module:      remotedisplay

    This class contains all methods to open an own shell and interacts with
    this. It also prints the verbose output.
    """

    TIMEOUT = 0.5 # sec.


    # -------------------------------------------------------------------------
    def __init__(self, parent, loghandler):
        # To create box drawings use ctrl shift u <xxxx> enter
        # for the drawing numbers have a look on:
        # https://jrgraphix.net/r/Unicode/2500-257F

        self.parent = parent
        self.loghandler = loghandler
        self.waitingForCallback = False


        self.curser = 0
        self.prompt = "cxmenu>"

        self.log = logging.getLogger('cxmenu.' + __name__)
        self.log.setLevel(logging.DEBUG)


        self.shell = curses.initscr()
        self.shell.timeout(100)
        curses.start_color()
        curses.curs_set(0)
        curses.noecho()
        curses.cbreak() # Enables input via keyboard without confirming enter.

        # Init color pairs
        curses.init_pair(1, curses.COLOR_RED, curses.COLOR_WHITE)
        curses.init_pair(2, curses.COLOR_WHITE, curses.COLOR_RED)
        curses.init_pair(3, curses.COLOR_GREEN, curses.COLOR_WHITE)

        # -------------------- Init timer for button colors -------------------
        self.endtime = t.time()

        # ---------------- Header and CX20xx display simulation ---------------
        self.shell.addstr(2, 0, ' ' * 70, curses.color_pair(2))
        self.shell.addstr(2, 2, "BECKHOFF BIOS API - by IgH Essen.",
                curses.color_pair(2))

        # Create white box
        for y in range(3, 9):
            self.shell.addstr(y, 0, ' ' * 70, curses.A_STANDOUT)

        # Create buttons
        self.initButtonSim()

        # Create information text
        self.shell.addstr(4, 32, 'To emulate the release of', curses.A_STANDOUT)
        self.shell.addstr(5, 32, 'enter press backspace.', curses.A_STANDOUT)


        # ------------------ Window for the terminal oputput ------------------
        self.hight = curses.LINES -10
        self.width = curses.COLS -1
        self.terminal = curses.newwin(self.hight, self.width, 10, 0)

        # --------------------------- Loginformation --------------------------
        self.log.debug('Started remote mode. To quit press \'q\'')


    # -------------------------------------------------------------------------
    def closeRemoteDisplay(self):
        curses.nocbreak()
        curses.echo()
        curses.endwin()



    # -------------------------------------------------------------------------
    def getShell(self):
        return self.shell


    # -------------------------------------------------------------------------
    def settimer(self, timeout):
        self.endtime = t.time() + timeout


    # -------------------------------------------------------------------------
    def setDryrunInfo(self):
        if opt.dryrun:
            self.shell.addstr(7, 32, '⇄ OFFLINE - No connection to CX21xx.',
                    curses.color_pair(1))

        else:
            self.shell.addstr(7, 32, '⇄ ONLINE',
                    curses.color_pair(3))


    # -------------------------------------------------------------------------
    def output(self, event = None):
        """Gets called if the main modul needs an output to the shell."""
        out = self.loghandler.readPuffer()
        for line in out:
            line = line.format()
            logMessage = line.split(' - ')

            if logMessage[0] == 'INFO':

                if logMessage[2].startswith('keyevent'):
                    # Create button simulator
                    btnEvent = logMessage[2].split(':')

                    if btnEvent[1] in ['65', 'UP']:
                        self.shell.addstr(4, 27, '▲', curses.color_pair(1))
                        self.settimer(self.TIMEOUT)

                    if btnEvent[1] in ['66', 'DOWN']:
                        self.shell.addstr(6, 27, '▼', curses.color_pair(1))
                        self.settimer(self.TIMEOUT)

                    if btnEvent[1] in ['67', 'RIGHT']:
                        self.shell.addstr(5, 29, '▶', curses.color_pair(1))
                        self.settimer(self.TIMEOUT)

                    if btnEvent[1] in ['68', 'LEFT']:
                        self.shell.addstr(5, 25, '◀', curses.color_pair(1))
                        self.settimer(self.TIMEOUT)

                    if btnEvent[1] in ['10', 'ENTER']:
                        self.shell.addstr(5, 27, '■', curses.color_pair(1))
                        self.settimer(self.TIMEOUT)

                    self.parent.registerCallback(self.output)
                    self.waitingForCallback = True
                    self.shell.refresh()

                else:
                    # Create display simulator
                    outstr = logMessage[2].split(' \\n ')

                    self.shell.addstr(4, 3, "┌────────────────┐", curses.A_STANDOUT)
                    self.shell.addstr(5, 3, "│                │", curses.A_STANDOUT)
                    self.shell.addstr(5, 4, outstr[0], curses.A_STANDOUT)
                    self.shell.addstr(6, 3, "│                │", curses.A_STANDOUT)
                    self.shell.addstr(6, 4, outstr[1], curses.A_STANDOUT)
                    self.shell.addstr(7, 3, "└────────────────┘", curses.A_STANDOUT)
                    self.shell.refresh()



            elif logMessage[0] == 'DEBUG' and opt.verbose:
                outstr = " " + self.prompt + " " + logMessage[2]
                self.terminal.addstr(self.curser, 0, outstr)
                self.terminal.refresh()

                if self.curser < self.hight - 2:
                    self.curser += 1
                else:
                    self.terminal.clear()
                    self.curser = 0


            if t.time() > self.endtime and self.waitingForCallback:
                self.initButtonSim()
                self.waitingForCallback = False
                self.parent.releaseCallback(self.output)


    # -------------------------------------------------------------------------
    def initButtonSim(self):
        self.shell.addstr(5, 27, '■', curses.A_STANDOUT)
        self.shell.addstr(4, 27, '▲', curses.A_STANDOUT)
        self.shell.addstr(5, 25, '◀', curses.A_STANDOUT)
        self.shell.addstr(5, 29, '▶', curses.A_STANDOUT)
        self.shell.addstr(6, 27, '▼', curses.A_STANDOUT)
        self.shell.refresh()



#
